package com.xxfc.platform.order.biz;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.github.wxiaoqi.security.common.biz.BaseBiz;
import com.github.wxiaoqi.security.common.enumconstant.LevelEnum;
import com.google.common.collect.Lists;
import com.xxfc.platform.order.contant.enumerate.StatisticsStatusEnum;
import com.xxfc.platform.order.entity.OrderMemberReceivedStatistics;
import com.xxfc.platform.order.mapper.OrderMemberReceivedStatisticsMapper;
import com.xxfc.platform.order.pojo.dto.OrderDTO;
import com.xxfc.platform.order.pojo.dto.OrderReceivedStatisticsFindDTO;
import lombok.RequiredArgsConstructor;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 会员订单统计
 *
 * @author libin
 * @email 18178966185@163.com
 * @date 2019-11-08 18:03:42
 */
@Service
@Transactional(rollbackFor = Exception.class)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class OrderMemberReceivedStatisticsBiz extends BaseBiz<OrderMemberReceivedStatisticsMapper, OrderMemberReceivedStatistics> {
    private final BaseOrderBiz baseOrderBiz;

    public List<OrderMemberReceivedStatistics> selectOrderReceivedStatistics(OrderReceivedStatisticsFindDTO orderReceivedStatisticsFindDTO) {
        return mapper.selectOrderMemberReceivedStatistics(orderReceivedStatisticsFindDTO);
    }

    public void orderMemberReceivedStatistics(Date date) {
        List<OrderMemberReceivedStatistics> orderMemberReceivedStatisticsList = new ArrayList<>(18);
        DateTime yesterday = DateUtil.yesterday();
        Date startDate = date == null ? DateUtil.beginOfDay(yesterday).toJdkDate() : DateUtil.beginOfDay(date);
        Date endDate = date == null ? DateUtil.endOfDay(yesterday).toJdkDate() : DateUtil.endOfDay(date);
        List<OrderDTO> orders = baseOrderBiz.selectOrdersByTypeAndTime(Arrays.asList(StatisticsStatusEnum.ORDER_TOUR_TYPE), startDate, endDate);

        //数据处理 把 完成状态6 取消状态2 的其他数据转换成状态为未完成3
        Map<String, Map<Integer, List<OrderDTO>>> ordersMap = orders.stream().peek(x -> {
            x.setStateGroup(String.format("%d-%d-%d-%d", StatisticsStatusEnum.DEFAULT_COMPANY, x.getOrderOrigin(), x.getPayWay()==null?StatisticsStatusEnum.NO_PAY_WAY:x.getPayWay(),x.getHasPay()));
        }).collect(Collectors.groupingBy(OrderDTO::getStateGroup, Collectors.groupingBy(OrderDTO::getMemberLevel, Collectors.toList())));

        if (!ordersMap.isEmpty()) {
            Set<Map.Entry<String, Map<Integer, List<OrderDTO>>>> orderSet = ordersMap.entrySet();
            for (Map.Entry<String, Map<Integer, List<OrderDTO>>> orderEntry : orderSet) {
                //获取状态
                String orderStatisticsGroup = orderEntry.getKey();
                Map<Integer, List<OrderDTO>> orderEntryValue = orderEntry.getValue();
                OrderMemberReceivedStatistics orderMemberReceivedStatistics = StatisticsStatusEnum.wrapStatisticsObject(startDate, orderStatisticsGroup, new OrderMemberReceivedStatistics());
                //1.普通会员
                List<OrderDTO> generalOrders = orderEntryValue.get(LevelEnum.GENERAL.getLevel());
                generalOrders = CollectionUtils.isEmpty(generalOrders) ? Collections.EMPTY_LIST : generalOrders;
                BigDecimal totalCommAmount = generalOrders.stream().map(OrderDTO::getRealAmount).reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
                orderMemberReceivedStatistics.setToalCommonAmmount(totalCommAmount);
                orderMemberReceivedStatistics.setTotalCommonQuantity(generalOrders.size());

                //2.黄金会员
                List<OrderDTO> goldOrders = orderEntryValue.get(LevelEnum.GOLD.getLevel());
                goldOrders = CollectionUtils.isEmpty(goldOrders) ? Collections.EMPTY_LIST : goldOrders;
                BigDecimal totalGoldAmount = goldOrders.stream().map(OrderDTO::getRealAmount).reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
                orderMemberReceivedStatistics.setTotalGoldAmount(totalGoldAmount);
                orderMemberReceivedStatistics.setTotalGoldQuantity(goldOrders.size());

                //3.钻石会员
                List<OrderDTO> diamondOrders = orderEntryValue.get(LevelEnum.DIAMOND.getLevel());
                diamondOrders = CollectionUtils.isEmpty(diamondOrders) ? Collections.EMPTY_LIST : diamondOrders;
                BigDecimal totalDiamondAmount = diamondOrders.stream().map(OrderDTO::getRealAmount).reduce(BigDecimal.ZERO, (x, y) -> x.add(y));
                orderMemberReceivedStatistics.setTotalDiamondAmmount(totalDiamondAmount);
                orderMemberReceivedStatistics.setTotalDiamondQuantity(diamondOrders.size());

                BigDecimal totalAmount = totalCommAmount.add(totalGoldAmount).add(totalDiamondAmount);
                Integer totalQuantity = orderMemberReceivedStatistics.getTotalCommonQuantity() + orderMemberReceivedStatistics.getTotalGoldQuantity() + orderMemberReceivedStatistics.getTotalDiamondQuantity();
                orderMemberReceivedStatistics.setTotalQuantity(totalQuantity);
                orderMemberReceivedStatistics.setTotalAmount(totalAmount);
                orderMemberReceivedStatisticsList.add(orderMemberReceivedStatistics);
            }
        }
        //创建剩余状态数据
        List stisticsActiveState = ordersMap.isEmpty() ? Collections.EMPTY_LIST : Lists.newArrayList(ordersMap.keySet());
        List<OrderMemberReceivedStatistics> otherStatisticsStateGroupList = createOtherStatisticsStateGroupList(startDate, stisticsActiveState, StatisticsStatusEnum.DEFAULT_COMPANY);
        orderMemberReceivedStatisticsList.addAll(otherStatisticsStateGroupList);
        //保存
        insertMemberReceivedStatisticsBatch(orderMemberReceivedStatisticsList);
    }

    /**
     * 创建剩余状态数据
     *
     * @param startDate
     * @param statisticsStateGroups
     * @param companyId
     * @return
     */
    private List<OrderMemberReceivedStatistics> createOtherStatisticsStateGroupList(Date startDate, List<String> statisticsStateGroups, Integer companyId) {
        List<OrderMemberReceivedStatistics> orderMemberReceivedStatisticsList = new ArrayList<>();
        OrderMemberReceivedStatistics orderMemberReceivedStatistics;
        List<String> otherStatisticsStateGroup = StatisticsStatusEnum.getOtherStatisticsStateGroup(Arrays.asList(companyId), statisticsStateGroups);
        OrderMemberReceivedStatistics orderMemberReceivedStatisticsClone = new OrderMemberReceivedStatistics();
        for (String stateGroup : otherStatisticsStateGroup) {
            orderMemberReceivedStatistics = StatisticsStatusEnum.wrapStatisticsObject(startDate, stateGroup, ObjectUtil.cloneByStream(orderMemberReceivedStatisticsClone));
            orderMemberReceivedStatistics.setTotalAmount(BigDecimal.ZERO);
            orderMemberReceivedStatistics.setTotalCommonQuantity(0);
            orderMemberReceivedStatistics.setToalCommonAmmount(BigDecimal.ZERO);
            orderMemberReceivedStatistics.setTotalGoldQuantity(0);
            orderMemberReceivedStatistics.setTotalGoldAmount(BigDecimal.ZERO);
            orderMemberReceivedStatistics.setTotalDiamondQuantity(0);
            orderMemberReceivedStatistics.setTotalDiamondAmmount(BigDecimal.ZERO);
            orderMemberReceivedStatisticsList.add(orderMemberReceivedStatistics);
        }
        return orderMemberReceivedStatisticsList;
    }

    public void insertMemberReceivedStatisticsBatch(List<OrderMemberReceivedStatistics> orderMemberReceivedStatistics) {
        mapper.insertList(orderMemberReceivedStatistics);
    }
}