package com.xxfc.platform.order.biz;

import com.github.wxiaoqi.security.common.biz.BaseBiz;
import com.google.common.collect.Lists;
import com.xxfc.platform.order.contant.enumerate.StatisticsStatusEnum;
import com.xxfc.platform.order.entity.OrderRentVehicleReceivedStatistics;
import com.xxfc.platform.order.mapper.OrderRentVehicleReceivedStatisticsMapper;
import com.xxfc.platform.order.pojo.account.OrderAccountBo;
import com.xxfc.platform.order.pojo.account.OrderAccountDetail;
import com.xxfc.platform.order.pojo.dto.OrderDTO;
import com.xxfc.platform.order.pojo.dto.OrderReceivedStatisticsFindDTO;
import com.xxfc.platform.vehicle.feign.VehicleFeign;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 租车订单统计
 *
 * @author libin
 * @email 18178966185@163.com
 * @date 2019-11-08 18:03:42
 */
@Service
@Transactional(rollbackFor = Exception.class)
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class OrderRentVehicleReceivedStatisticsBiz extends BaseBiz<OrderRentVehicleReceivedStatisticsMapper, OrderRentVehicleReceivedStatistics> {

    private final BaseOrderBiz baseOrderBiz;
    private final OrderAccountBiz orderAccountBiz;
    private final VehicleFeign vehicleFeign;
    private final int PAY_ORDER=101;

    public List<OrderRentVehicleReceivedStatistics> selectOrderReceivedStatistics(OrderReceivedStatisticsFindDTO orderReceivedStatisticsFindDTO) {
        return mapper.selectOrderRentVehicleReceivedStatistics(orderReceivedStatisticsFindDTO);
    }

    public void orderRentVehicleReceivedStatistics(Date startDate,Date endDate) {
        List<OrderRentVehicleReceivedStatistics> orderRentVehicleReceivedStatisticsList = new ArrayList<>();


        List<OrderDTO> orderDTOS = baseOrderBiz.selectOrdersByTypeAndTime(Arrays.asList(StatisticsStatusEnum.ORDER_RENT_VEHICLE_TYPE), startDate, endDate);

        //订单账目信息
      //  List<OrderAccountBo> orderAccountBoList = orderAccountBiz.selectByDate(startDate,endDate);
        //数据处理 把 未支付的单的支付方式 设置为 99
        Map<String, List<OrderDTO>> ordersMap = orderDTOS.stream().peek(x -> {
            x.setStateGroup(String.format("%d-%d-%d-%d", x.getCompanyId(), x.getOrderOrigin(), x.getPayWay() == null ? StatisticsStatusEnum.NO_PAY_WAY : x.getPayWay(), x.getStatus()));
        }).collect(Collectors.groupingBy(OrderDTO::getStateGroup, Collectors.toList()));

        if (!ordersMap.isEmpty()) {
            Set<Map.Entry<String, List<OrderDTO>>> ordersSet = ordersMap.entrySet();
            for (Map.Entry<String, List<OrderDTO>> orderEntry : ordersSet) {
                String orderKey = orderEntry.getKey();
                List<OrderDTO> orderDTOList = orderEntry.getValue();
                OrderRentVehicleReceivedStatistics orderRentVehicleReceivedStatistics = StatisticsStatusEnum.wrapStatisticsObject(startDate, orderKey, new OrderRentVehicleReceivedStatistics());
               /* BigDecimal totalAmount = BigDecimal.ZERO;
                BigDecimal refundAmount = BigDecimal.ZERO;
                for (OrderAccountBo orderAccountBo : orderAccountBos) {
                    OrderAccountDetail accountDetailEntity = orderAccountBo.getAccountDetailEntity();
                    if (orderAccountBo.getAccountType() == PAY_ORDER) {
                        totalAmount.add(accountDetailEntity.getOrderAmount()).add(accountDetailEntity.getDepositAmount());
                    } else {
                        refundAmount.add(accountDetailEntity.getOrderAmount()).add(accountDetailEntity.getDepositAmount());
                    }
                }
                orderRentVehicleReceivedStatistics.setTotalAmount(totalAmount);
                long size = orderAccountBos.stream().map(OrderDTO::getId).distinct().count();
                orderRentVehicleReceivedStatistics.setTotalQuantity(Integer.valueOf(String.valueOf(size)));*/
            }
        }
        List<Integer> companyIds = vehicleFeign.findCompanyIdsByAreaId(null);
        List stisticsActiveState = ordersMap.isEmpty() ? Collections.EMPTY_LIST : Lists.newArrayList(ordersMap.keySet());
        List<OrderRentVehicleReceivedStatistics> otherStatisticsStateGroupList = createOtherStatisticsStateGroupList(startDate,stisticsActiveState,companyIds);
        orderRentVehicleReceivedStatisticsList.addAll(otherStatisticsStateGroupList);
        insertMemberReceivedStatisticsBatch(orderRentVehicleReceivedStatisticsList);
    }

    /**
     * 创建剩余状态数据
     *
     * @param startDate
     * @param statisticsStateGroups
     * @param companyIds
     * @return
     */
    private List<OrderRentVehicleReceivedStatistics> createOtherStatisticsStateGroupList(Date startDate, List<String> statisticsStateGroups, List<Integer> companyIds) {
        List<OrderRentVehicleReceivedStatistics> orderRentVehicleReceivedStatisticsList = new ArrayList<>();
        List<String> otherStatisticsStateGroup = StatisticsStatusEnum.getOtherStatisticsStateGroup(companyIds, statisticsStateGroups);
        List<OrderRentVehicleReceivedStatistics> result = otherStatisticsStateGroup.parallelStream().map(stateGroup->{
            OrderRentVehicleReceivedStatistics orderRentVehicleReceivedStatistics = StatisticsStatusEnum.wrapStatisticsObject(startDate, stateGroup, new OrderRentVehicleReceivedStatistics());
            orderRentVehicleReceivedStatistics.setTotalAmount(BigDecimal.ZERO);
            orderRentVehicleReceivedStatistics.setTotalQuantity(0);
            return  orderRentVehicleReceivedStatistics;
        }).collect(Collectors.toList());
        orderRentVehicleReceivedStatisticsList.addAll(result==null?Collections.EMPTY_LIST:result);
        return orderRentVehicleReceivedStatisticsList;
    }
    public void insertMemberReceivedStatisticsBatch(List<OrderRentVehicleReceivedStatistics> orderRentVehicleReceivedStatistics) {
        mapper.insertList(orderRentVehicleReceivedStatistics);
    }
}